#include <MDS_misc.h>
#include <MDS_embedding.h>
#include <MDS_Kruskal.h>
#include <OKC_data.h>
#include "som.h"

#include <stdio.h>

/*
 * argument descriptions:
 * argv[1]-original .okc file
 * argv[2]-generated som mds space .okc file
 * argv[3]-number of embedding dimensions in mds
 * argv[4]-logfile for mds stress
 *
 */

int main (int argc, char *argv[])
{
	if (argc==1 || argc==2) {
		cout<<"Parameter requirements:"<<endl;
		cout<<"  argv[1]:original .okc data file (required),"<<endl;
		cout<<"  argv[2]:derived .okc file in reduced dimension spaces (required)"<<endl;
		cout<<"  argv[3]:number of dimensions for MDS (optional, integer, default=2)"<<endl;
		return 1;
	}

	if ((argc>4 && argc<15) || argc>15) {
		cout<<"Parameter requirements:"<<endl;
		cout<<"  argv[1]:original .okc data file (required),"<<endl;
		cout<<"  argv[2]:derived .okc file in reduced dimension spaces (required)"<<endl;
		cout<<"  argv[3]:number of dimensions for MDS (optional, integer, default=2)"<<endl;

		cout<<"\targv[4]:topology type (required: integer, 0-rectangle; 1-hextangle)"<<endl;
		cout<<"\targv[5]:neighborhood function type (required: integer: 0-bubble; 1-gaussian)"<<endl;
		cout<<"\targv[6]:number of grid in x-dimension (required)"<<endl;
		cout<<"\targv[7]:number of grid in y-dimension (required)"<<endl;

		cout<<"\targv[8]:alpha type (required: short, 0-linear, 1-inverse_t)"<<endl;
		cout<<"\targv[9]:radius (required: integer)"<<endl;
		cout<<"\targv[10]:initial alpha value (required: double)"<<endl;
		cout<<"\targv[11]:run length (requried: integer)"<<endl;

		cout<<"\targv[12]:radius in refine step (required: integer)"<<endl;
		cout<<"\targv[13]:initial alpha value in refine step (required: double)"<<endl;
		cout<<"\targv[14]:run length in refine step (requried: integer)"<<endl;

		return 1;
	}


	ifstream is;
	is.open (argv[1]);
	if (is==NULL) {
		cout<<"cannot open okc file"<<endl;
		return 1;
	}

	OKC_data iris;
	is>>iris;
	is.close ();

	MDS_Matrix raw_data=iris.data;
	MDS_Matrix centered=raw_data.col_center();

	MDS_Matrix dissim=centered.row_p2sqr_dist();


	MDS_Kruskal emd;
	if (argc>3) emd.setDimensionNumber(atoi(argv[3]));
	emd.assignDissim (dissim);
	emd.classicalScaling (dissim);


	// Multidimensional scaling
	/*
	ofstream logfile;
	logfile.open ("logfile.txt");
	if (logfile==NULL) {
		cout<<"canont open log file data"<<endl;
		return 1;
	}
	*/
	
	emd.init ();
	emd.iterateOnce ();

	cout<<"Mulitdimensional scaling is in the progress..."<<endl;
	for (int i=0; i<100;i++) {
		double ss2=emd.ss2();

		//cout<<endl<<"ith time:"<<i<<endl;
		//cout<<"ss2="<<ss2<<endl;
		//logfile<<endl<<"ith time:"<<i<<endl;
		//logfile<<"ss2="<<ss2<<endl;

		emd.iterateOnce ();
		if(fabs(ss2-emd.ss2())<0.001) break;
		//emd.printStress();
	}
	cout<<"MDS done!"<<endl;
	//logfile.close ();


	/* SOM algorithms
	 * void randinit(const SOM_Matrix& data, int topoltype, int nei, int xdim, int ydim);
	 * void training(short alpha_type, double radius, double alpha, long length);
	 */
	SOM som;
	cout<<"Self-organizing mapping is in the progress..."<<endl;
	if (argc<4) {
		som.randinit (raw_data, 0, 0, 8, 12);
		som.training (0, 10, 0.05, 1000);
		som.training (0, 3, 0.02, 10000);
	}
	if (argc==15) {
		som.randinit (raw_data, 0, atoi(argv[5]), atoi(argv[6]), atoi(argv[7]));
		//logfile<<"after initialization"<<endl;
		//logfile<<som<<endl;

		som.training (atoi(argv[8]), atoi(argv[9]), atof(argv[10]), atol(argv[11]));
		//logfile<<"after 1st training"<<endl;
		//logfile<<som<<endl;

		som.training (atoi(argv[8]), atoi(argv[12]), atof(argv[13]), atol(argv[14]));
		//logfile<<"after 2nd training"<<endl;
		//logfile<<som<<endl;

	}
	//logfile.close ();

	cout<<"SOM done!"<<endl;


	OKC_data okc_mds, okc_som;
	okc_mds.create_okc_data (emd.Xik, "mds");
	okc_som.create_okc_data (som.mapped_coord (raw_data), "som");

	OKC_data agreg, mds_som;
	mds_som=okc_mds.combine (okc_som);
	agreg=iris.combine (mds_som);

	ofstream mdsom;
	mdsom.open (argv[2]);
	if (mdsom==NULL) {
		cout<<"canont open mds_som_okc data"<<endl;
		return 1;
	}

	mdsom<<agreg<<endl;
	mdsom.close ();

	return 0;
}